library dashstrap_button;

import 'dart:math';

import 'package:flutter/material.dart';

enum DSButtonSize { small, normal, large, fullwidth }
enum DSButtonFeedbackStyle { normal, opacity }
enum DSButtonFontStyle { normal, strong }
enum DSButtonStyle { normal, outline }

class DSButton extends StatefulWidget {
  final DSButtonSize size;
  final DSButtonFeedbackStyle feedbackStyle;
  final String text;
  final double? fontSize;
  final DSButtonFontStyle fontStyle;
  final Color fontColor;
  final Color? color;
  final IconData? icon;
  final Function(TapUpDetails)? onTapUp;
  final Function(TapDownDetails)? onTapDown;
  final DSButtonStyle style;
  final double activeOpacity;
  final bool disabled;
  final Color? disabledColor;

  DSButton({
    required this.text,
    this.size = DSButtonSize.normal,
    this.feedbackStyle = DSButtonFeedbackStyle.normal,
    this.fontSize,
    this.fontStyle = DSButtonFontStyle.normal,
    this.fontColor = Colors.white,
    this.color,
    this.icon,
    this.onTapUp,
    this.onTapDown,
    this.style = DSButtonStyle.normal,
    this.activeOpacity = 0.3,
    this.disabled = false,
    this.disabledColor,
    Key? key,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _DSButtonState();
  }
}

class _DSButtonState extends State<DSButton> {
  late Color defaultBackgroundColor;
  late Color currentBackgroundColor;
  late double defaultOpacity;
  late double currentOpacity;

  @override
  void didChangeDependencies() {
    defaultBackgroundColor = getBackgroundColor();
    currentBackgroundColor = defaultBackgroundColor;
    defaultOpacity = 1;
    currentOpacity = 1;
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, boxconstraints) {
      return GestureDetector(
        onTapDown: onTapDown,
        onTapUp: onTapUp,
        child: AnimatedOpacity(
          opacity: currentOpacity,
          duration: Duration(milliseconds: 150),
          child: AnimatedContainer(
            duration: Duration(milliseconds: 200),
            decoration: BoxDecoration(
              color: widget.style == DSButtonStyle.outline
                  ? Colors.transparent
                  : currentBackgroundColor,
              borderRadius: BorderRadius.circular(getBorderRadius()),
              border: Border.all(
                width: widget.style == DSButtonStyle.outline ? 1 : 0,
                color: widget.style == DSButtonStyle.outline
                    ? currentBackgroundColor
                    : Colors.transparent,
              ),
            ),
            width: widget.size == DSButtonSize.fullwidth
                ? boxconstraints.maxWidth
                : null,
            padding: getButtonPadding(),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              mainAxisSize: widget.size == DSButtonSize.fullwidth
                  ? MainAxisSize.max
                  : MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                getIcon() ?? Container(height: 0, width: 0),
                Text(
                  widget.text,
                  style: TextStyle(
                    fontSize: getFontSize(),
                    fontWeight: widget.fontStyle == DSButtonFontStyle.strong
                        ? FontWeight.bold
                        : FontWeight.w500,
                    color: getTextColor(),
                  ),
                ),
              ],
            ),
          ),
        ),
      );
    });
  }

  Color getTextColor() {
    if (widget.disabled) {
      if (widget.style == DSButtonStyle.outline) {
        return widget.disabledColor ?? Color(0xFFEAEAEA);
      }

      return Colors.white;
    }

    if (widget.style == DSButtonStyle.outline) {
      return currentBackgroundColor;
    }

    return widget.fontColor;
  }

  Widget? getIcon() {
    if (widget.icon == null) {
      return null;
    }

    double rightPadding = 16;
    double iconSizeDelta = 4;

    if (widget.size == DSButtonSize.small) {
      rightPadding = 8;
      iconSizeDelta = 2;
    }

    return Padding(
      padding: EdgeInsets.only(right: rightPadding),
      child: Icon(
        widget.icon,
        color: getTextColor(),
        size: getFontSize() + iconSizeDelta,
      ),
    );
  }

  double getBorderRadius() {
    switch (widget.size) {
      case DSButtonSize.small:
        return 4;
      default:
        return 6;
    }
  }

  Color getBackgroundColor() {
    if (widget.disabled) {
      return widget.disabledColor ?? Color(0xFFEAEAEA);
    }

    if (widget.color != null) {
      return widget.color!;
    }

    return Theme.of(context).accentColor;
  }

  EdgeInsets getButtonPadding() {
    double horizontal = 8;
    switch (widget.size) {
      case DSButtonSize.small:
        break;
      case DSButtonSize.normal:
        horizontal = horizontal * 2;
        break;
      case DSButtonSize.large:
        horizontal = horizontal * 2.5;
        break;
      case DSButtonSize.fullwidth:
        horizontal = horizontal * 2;
        break;
    }

    return EdgeInsets.only(
      left: horizontal,
      right: horizontal,
      top: horizontal - 4,
      bottom: horizontal - 4,
    );
  }

  double getFontSize() {
    if (widget.fontSize != null) {
      return widget.fontSize!;
    }

    switch (widget.size) {
      case DSButtonSize.small:
        return 12;
      case DSButtonSize.normal:
        return 14;
      case DSButtonSize.large:
        return 16;
      case DSButtonSize.fullwidth:
        return 16;
    }
  }

  void onTapDown(TapDownDetails tapDownDetails) {
    if (widget.disabled) {
      return;
    }

    Color activeColor = Color.fromARGB(
      defaultBackgroundColor.alpha,
      max(defaultBackgroundColor.red - 30, 0),
      max(defaultBackgroundColor.green - 30, 0),
      max(defaultBackgroundColor.blue - 30, 0),
    );

    if (widget.feedbackStyle == DSButtonFeedbackStyle.opacity) {
      setState(() {
        currentOpacity = widget.activeOpacity;
      });
    } else {
      setState(() {
        currentBackgroundColor = activeColor;
      });
    }

    if (widget.onTapDown != null) {
      widget.onTapDown!(tapDownDetails);
    }
  }

  void onTapUp(TapUpDetails tapUpDetails) {
    if (widget.disabled) {
      return;
    }
    setState(() {
      currentOpacity = defaultOpacity;
      currentBackgroundColor = defaultBackgroundColor;
    });

    if (widget.onTapUp != null) {
      widget.onTapUp!(tapUpDetails);
    }
  }
}
