import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_test/flutter_test.dart';

import 'package:dashstrap_button/dashstrap_button.dart';

void main() {
  Color _getActiveColor(Color baseColor) {
    int red = max(baseColor.red - 30, 0);
    int green = max(baseColor.green - 30, 0);
    int blue = max(baseColor.blue - 30, 0);

    return Color.fromRGBO(red, green, blue, 1);
  }

  testWidgets("Test that default button renders correctly", (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            color: Color(0xFF1890FF),
          ),
        ),
      ),
    );

    Finder gestureFinder = find.byType(GestureDetector);
    Finder containerFinder = find.byType(AnimatedContainer);
    Finder opacityFinder = find.byType(AnimatedOpacity);
    Finder rowFinder = find.byType(Row);
    Finder textFinder = find.text(text);

    // Check that the right widgets render the right number of times
    expect(gestureFinder, findsOneWidget);
    expect(opacityFinder, findsOneWidget);
    expect(rowFinder, findsOneWidget);
    expect(textFinder, findsOneWidget);
    expect(containerFinder, findsNWidgets(1));

    // Check that background of default button is the default color
    AnimatedContainer container =
        containerFinder.last.evaluate().single.widget as AnimatedContainer;
    expect((container.decoration as BoxDecoration).color,
        equals(Color(0xFF1890FF)));
  });

  testWidgets("Test that button with icon renders correctly", (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            color: Color(0xFF1890FF),
            icon: Icons.email,
          ),
        ),
      ),
    );

    Finder gestureFinder = find.byType(GestureDetector);
    Finder containerFinder = find.byType(AnimatedContainer);
    Finder opacityFinder = find.byType(AnimatedOpacity);
    Finder rowFinder = find.byType(Row);
    Finder textFinder = find.text(text);
    Finder iconFinder = find.byType(Icon);

    // Check that the right widgets render the right number of times
    expect(gestureFinder, findsOneWidget);
    expect(opacityFinder, findsOneWidget);
    expect(rowFinder, findsOneWidget);
    expect(textFinder, findsOneWidget);
    expect(containerFinder, findsNWidgets(1));
    expect(iconFinder, findsOneWidget);

    // Check that background of default button is the default color
    AnimatedContainer container =
        containerFinder.last.evaluate().single.widget as AnimatedContainer;
    expect((container.decoration as BoxDecoration).color,
        equals(Color(0xFF1890FF)));
  });

  testWidgets("Test that key provided is assigned to the root widget",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            key: Key("TestKey"),
          ),
        ),
      ),
    );

    Finder containerFinder = find.byType(DSButton);
    DSButton root = containerFinder.first.evaluate().single.widget as DSButton;
    expect(root.key, equals(Key("TestKey")));
  });

  testWidgets("Test that provided font size property is applied",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            fontSize: 26,
          ),
        ),
      ),
    );

    Finder textFinder = find.text(text);
    Text textWidget = textFinder.first.evaluate().single.widget as Text;
    expect(textWidget.style?.fontSize, equals(26));
  });

  testWidgets("Test that provided font color property is applied",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            fontColor: Colors.red,
            color: Colors.blue,
          ),
        ),
      ),
    );

    Finder textFinder = find.text(text);
    Text textWidget = textFinder.first.evaluate().single.widget as Text;
    expect(textWidget.style?.color, equals(Colors.red));
  });

  testWidgets("Test that on pressing the button the background color changes",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            color: Color(0xFF1890FF),
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.text(text)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedContainer);
    AnimatedContainer container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedContainer;
    expect((container.decoration as BoxDecoration).color,
        equals(_getActiveColor(Color(0xFF1890FF))));
  });

  testWidgets(
      "Test that for feedback style opacity, on press the opacity changes accordingly",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            feedbackStyle: DSButtonFeedbackStyle.opacity,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.text(text)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedOpacity);
    AnimatedOpacity container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedOpacity;
    expect(container.opacity, equals(0.3));
  });

  testWidgets(
      "Test that for feedback style opacity, on press the opacity changes accordingly when using custom active opacity",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            feedbackStyle: DSButtonFeedbackStyle.opacity,
            activeOpacity: 0.5,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.text(text)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedOpacity);
    AnimatedOpacity container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedOpacity;
    expect(container.opacity, equals(0.5));
  });

  testWidgets("Test that tap down and up are called, and in the correct order",
      (tester) async {
    // The widget uses futures, without runAsync the test will throw a pending timer error
    await tester.runAsync(() async {
      String text = "Test";
      List<String> tapResult = [];
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSButton(
              text: text,
              onTapDown: (_) {
                tapResult.add("down");
              },
              onTapUp: (_) {
                tapResult.add("up");
              },
            ),
          ),
        ),
      );

      await tester.tap(find.text(text));
      await tester.pump();

      expect(tapResult.length, equals(2));
      expect(tapResult[0], equals("down"));
      expect(tapResult[1], equals("up"));
    });
  });

  testWidgets("Test that button color changes properly when disabled",
      (tester) async {
    await tester.runAsync(() async {
      String text = "Test";
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSButton(
              text: text,
              disabled: true,
            ),
          ),
        ),
      );

      Finder containerFinder = find.byType(AnimatedContainer);
      expect(containerFinder, findsOneWidget);

      AnimatedContainer container =
          containerFinder.last.evaluate().single.widget as AnimatedContainer;
      expect((container.decoration as BoxDecoration).color,
          equals(Color(0xFFEAEAEA)));
    });
  });

  testWidgets("Test that custom diabled color is used when disabled",
      (tester) async {
    await tester.runAsync(() async {
      String text = "Test";
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSButton(
              text: text,
              disabled: true,
              disabledColor: Colors.red,
            ),
          ),
        ),
      );

      Finder containerFinder = find.byType(AnimatedContainer);
      expect(containerFinder, findsOneWidget);

      AnimatedContainer container =
          containerFinder.last.evaluate().single.widget as AnimatedContainer;
      expect((container.decoration as BoxDecoration).color, equals(Colors.red));
    });
  });

  testWidgets(
      "Test that for feedback style opacity, on press the opacity does not change when disabled",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            feedbackStyle: DSButtonFeedbackStyle.opacity,
            disabled: true,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.text(text)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedOpacity);
    AnimatedOpacity container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedOpacity;
    expect(container.opacity, equals(1));
  });

  testWidgets(
      "Test that on pressing the button the background color does not change when disabled",
      (tester) async {
    String text = "Test";
    await tester.pumpWidget(
      MaterialApp(
        home: Material(
          child: DSButton(
            text: text,
            disabledColor: Color(0xFF1890FF),
            disabled: true,
          ),
        ),
      ),
    );

    await tester.startGesture(tester.getCenter(find.text(text)));
    await tester.pump();

    Finder animatedContainerFinder = find.byType(AnimatedContainer);
    AnimatedContainer container = animatedContainerFinder.last
        .evaluate()
        .single
        .widget as AnimatedContainer;
    expect((container.decoration as BoxDecoration).color,
        equals(Color(0xFF1890FF)));
  });

  testWidgets("Test that tap down and up are not called when disabled",
      (tester) async {
    // The widget uses futures, without runAsync the test will throw a pending timer error
    await tester.runAsync(() async {
      String text = "Test";
      List<String> tapResult = [];
      await tester.pumpWidget(
        MaterialApp(
          home: Material(
            child: DSButton(
              disabled: true,
              text: text,
              onTapDown: (_) {
                tapResult.add("down");
              },
              onTapUp: (_) {
                tapResult.add("up");
              },
            ),
          ),
        ),
      );

      await tester.tap(find.text(text));
      await tester.pump();

      expect(tapResult.length, equals(0));
    });
  });
}
